using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using XNACS1Lib;

/// Information on Resoruces:
/// 
/// Images: All images download from: Microsoft Office Clipart: http://office.microsoft.com/en-us/images/
///         format of images may have been converted.
/// 
/// Audio Effect: Recrord by Kelvin Sung.
/// 
/// Background audio: BackgroundMusic.mp3
///     Extracted from: Somewhere Sunny (ver 2).mp3
///     Downloaded from: incompetech.com
///     Original Composer: Kevin MacLeod
///     


namespace ClassExample
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class ClassExample : XNACS1Base
    {
        // Variables should be declared here
        // and initialized in "InitializedWorld()"

        #region Declare the paddles and speed
        XNACS1Rectangle leftPaddle;
        XNACS1Rectangle rightPaddle;

        // variable for horizontal speed
        float speedX;

        // variable for verticle speed
        float speedY;
        #endregion

        XNACS1Circle a, b, c, d, e, f;    


        /// <summary>
        /// Initialize the internal state of your game
        /// </summary>
        protected override void InitializeWorld()
        {
            // Default window size is 800x600
            #region Initialize the window and coordinate
            // const float kBgAudioLevel = 0.1f;   // Default Audio level
            const int kWindowWidth = 800;       // Default Window Width
            const int kWindowHeight = 600;      // Default Window Height

            // PlayBackgroundAudio("BackgroundMusic", kBgAudioLevel);          // Background audio
            SetAppWindowPixelDimension(false, kWindowWidth, kWindowHeight); // Not full screen
            World.SetWorldCoordinate(Vector2.Zero, kWindowWidth);           // Each pixel is one unit
            #endregion
            World.SetBackgroundColor(Color.LightGray);

            #region initialize the paddles
            // define the paddles
            leftPaddle = new XNACS1Rectangle();
            leftPaddle.CenterX = 30f;
            leftPaddle.CenterY = 300f;
            leftPaddle.Width = 15f;
            leftPaddle.Height = 100f;
            leftPaddle.Color = Color.Brown;
            leftPaddle.Label = "Left\nPaddle";

            rightPaddle = new XNACS1Rectangle();
            rightPaddle.CenterX = 770f;
            rightPaddle.CenterY = 300f;
            rightPaddle.Width = 15f;
            rightPaddle.Height = 100f;
            rightPaddle.Color = Color.Pink;
            rightPaddle.Label = "Right\nPaddle";
            #endregion
            // This is to hide details
            CreateBullseyeAt(200, 300);

            float x = RandomFloat(-8f, 8f);
            float y = RandomFloat(-1f, 1f);
            // This is to hide details and for reuse
            SetRandomSpeed(x, y);
        }

        /// <summary>
        /// Update the internal state of your game
        /// </summary>
        protected override void UpdateWorld()
        {          
            #region Quit: if the back-button ("F1") is pressed
            if (GamePad.ButtonBackClicked())
                Exit();
            #endregion 

            // Move the Left/Right Paddles by thumbSticks
            MoveThePaddles();

            // move bullseye by speedX and speedY
            MoveBullseyeBy(speedX, speedY);

            if (CollideWithBullseye(leftPaddle) || CollideWithBullseye(rightPaddle)) 
            {
                speedX *= -1;            
            }

            if (GamePad.ButtonAClicked())
            {
                ResetBullseyeTo(400, 300);
            }

            // goes out of bonud left/right
            if ((a.CenterX < 0) || (a.CenterX > 800))
            {
                speedX *= -1;
                ResetBullseyeTo(RandomFloat(300, 500), RandomFloat(200, 400));
            }

            // touch top/bottom
            if ((a.CenterY < 0) || (a.CenterY > 600))
            {
                speedY *= -1;
            }

            EchoToBottomStatus("Bullseye at:" + f.Center);
        }

        // Paddle functions
        void MoveThePaddles()
        {
            leftPaddle.CenterY += GamePad.ThumbSticks.Left.Y * 5f;
            rightPaddle.CenterY += GamePad.ThumbSticks.Right.Y * 5f;
        }

        // speed functroins
        void SetRandomSpeed(float sx, float sy)
        {
            speedX = sx;
            speedY = sy;
        }

        #region  bullseye functions
        void CreateBullseyeAt(float xPos, float yPos)
        {
            // outter most
            a = new XNACS1Circle(new Vector2(xPos, yPos), 25);
            a.Color = new Color(255, 0, 0);

            // moving to smaller ones
            b = new XNACS1Circle(new Vector2(xPos, yPos), 20);
            b.Color = new Color(50, 255, 50);

            c = new XNACS1Circle(new Vector2(xPos, yPos), 15);
            c.Color = new Color(100, 100, 255);

            d = new XNACS1Circle(new Vector2(xPos, yPos), 12);
            d.Color = new Color(255, 150, 150);

            e = new XNACS1Circle(new Vector2(xPos, yPos), 8);
            e.Color = new Color(200, 255, 200);

            f = new XNACS1Circle(new Vector2(xPos, yPos), 5);
            f.Color = new Color(250, 250, 255); 
        }

        void ResetBullseyeTo(float xPos, float yPos)
        {
            a.CenterX = xPos;
            a.CenterY = yPos;
            b.CenterX = xPos;
            b.CenterY = yPos;
            c.CenterX = xPos;
            c.CenterY = yPos;
            d.CenterX = xPos;
            d.CenterY = yPos;
            e.CenterX = xPos;
            e.CenterY = yPos;
            f.CenterX = xPos;
            f.CenterY = yPos;

            // Reset the bullseye position and new random speed
            SetRandomSpeed(RandomFloat(-10, 10), RandomFloat(-2, 2));
        }

        void MoveBullseyeBy(float dx, float dy)
        {
            a.CenterX += dx;
            a.CenterY += dy;
            b.CenterX += dx;
            b.CenterY += dy;
            c.CenterX += dx;
            c.CenterY += dy;
            d.CenterX += dx;
            d.CenterY += dy;
            e.CenterX += dx;
            e.CenterY += dy;
            f.CenterX += dx;
            f.CenterY += dy;
        }

        bool CollideWithBullseye(XNACS1Rectangle aPaddle)
        {
            return aPaddle.Collided(a);
        }
        #endregion
    }
}